// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *	Reliable Broadcast over unreliable low-bandwidth (satellite)
 *			channels with no feedback
 *
 * 		Link Access Protocol - a File Link
 *
 * This is a particular type of link access via UNIX file (which can be
 * a pipe, btw). The name of the file is determined from a env
 * variable LAPFILENAME
 * Note, the file specified in LAPFILENAME may as well be a /dev/tty or
 * /dev/cua/a etc., a serial port that is. In that case, it's entirely a
 * user responsibility to set that /dev/tty (/dev/cua) appropriately, that
 * is, set a baud rate, handshaking protocol, etc. At the very least, one
 * has to
 *	stty raw -echo < /dev/cua
 * before calling a broadcaster/receiver; after the program is finished, one
 * might want to restore the cooked mode
 *	stty -raw echo < /dev/cua
 * if, of course, one needs that /dev/cua as a terminal.
 *
 * $Id: LAPFile.h,v 1.1 1996/07/30 14:29:58 oleg Exp oleg $
 *
 ************************************************************************
 */

#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif

#ifndef __LAPFile_h
#define __LAPFile_h

#include "LAP.h"

class LAPFileOut : public LAPOut
{
  FILE * fp;

public:
  LAPFileOut(void);
  virtual ~LAPFileOut(void);
  
  enum { SOH = 'A' };		// Start of the packet char

  				// Standard LAPOut interface
  virtual void write_byte(const unsigned char a_byte);
  virtual void begin_packet(void);
  virtual void finish_packet(void);
};

class LAPFileIn : public LAPIn
{
  FILE * fp;
  Random::One_in_n error_event;
  int get_char(void);
  
public:
  LAPFileIn(void);
  virtual ~LAPFileIn(void);
  				// Standard LAPIn interface
  virtual unsigned char read_byte(void);
  virtual void look_for_start_of_packet(void);
  virtual void ending_packet(void);
};
#endif
