// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *	Reliable Broadcast over unreliable low-bandwidth (satellite)
 *			channels with no feedback
 *
 * 		Link Access Protocol - a File Link
 *
 * This is a particular type of link access via Windoze communication
 * port. The name of the port is determined from a env
 * variable LAPSERPORT. It is assumed that the port is already
 * set up for desirable baud rate, parity, etc. This program doesn't
 * do any of the setup, use 'mode' or other configurational utility.
 * However, Windoze95 turns out to unable to deal with the serial
 * port settings made by its own configuration utilities. For example,
 * even when COM1 is set to 300 baud through the Control panel, even
 * if Windows.INI and System.INI (checked by running 'sysedit') all
 * contain the correct settings for COM1, even if I use
 * mode com1: 300,n,8,1 - the serial port's speed is somehow set
 * to 1200 baud (as DCB.BaudRate after GetCommState() reveals, see below).
 * So, we got to manually set up the correct port's speed once again...
 * For that purpose, an env variable LAPBAUDRATE has to be specified
 * to tell the right baud rate for the serial port. Note, since 'mode'
 * command on WinNT does all the right things as to the comm port settings,
 * the use of LAPBAUDRATE under WinNT is _unnecessary_.
 *
 * In addition, an env variable LAPTIMEOUT may be specified to tell
 * the default timeout per character (in ticks, that is, 1/60th of a second).
 * E.g., if LAPTIMEOUT=60, a reading of a packet would be aborted (timed out)
 * if no byte can be received within 1 second. Still, waiting for the
 * start of the packet is done without a timeout.
 * The default value for this parameter is 300 (that is, 5 seconds).
 *
 * $Id: LAPSWinI.cc,v 1.3 1996/12/06 22:06:44 oleg Exp oleg $
 *
 ************************************************************************
 */

#ifdef __GNUC__
#pragma implementation
#endif

#include "util.h"
#include "LAPSWin.h"
#include <winbase.h>
//#include <windowsx.h>

static const char LAPPort_name [] = "LAPSERPORT";
static const char LAPTimeout [] = "LAPTIMEOUT";
static const char LAPBaudRate [] = "LAPBAUDRATE";

				// Get timout value (in ticks)
static int get_timeout(void)
{
  const int period = atoi(xgetenv(LAPTimeout,"300"));
  Logger::log("byte receive timeout is set to %d ticks",period);
  assert( period > 0 );
  return period;
}

			// Determine the name of the LAP In File and open it
LAPSWinIn::LAPSWinIn(void)
	: timeout((1000L*get_timeout())/60)
{
  const char * port_name = xgetenv(LAPPort_name,0);
  if( (comdev_handle =
        CreateFile(port_name,GENERIC_READ,0,NULL,OPEN_EXISTING,
                   FILE_FLAG_NO_BUFFERING,NULL)) ==
        INVALID_HANDLE_VALUE )
    perror("opening LAPPort"),
    _error("Failed to open LAPPort '%s' due to last error %d",
           port_name,GetLastError());
  COMMTIMEOUTS timeouts = {0,0,0,0,0};   		// Means wait forever
  assert( SetCommTimeouts(comdev_handle,&timeouts) );
  DCB dcb;
  //assert( BuildCommDCB("baud=300 parity=N data=8 stop=1",&dcb) );
  assert( GetCommState(comdev_handle,&dcb) );
  dcb.fNull=0; dcb.fBinary=1; dcb.fParity=0;
  dcb.fDtrControl = DTR_CONTROL_DISABLE;
  dcb.fDsrSensitivity=0;
  dcb.fInX=0;
  dcb.fErrorChar=0;
  dcb.fRtsControl=RTS_CONTROL_DISABLE;
  dcb.Parity=NOPARITY;
  if( int baud_rate = atoi(xgetenv(LAPBaudRate,"0")) )
    dcb.BaudRate = baud_rate,
    Logger::log("baud rate is set up to %d",baud_rate);
  assert( SetCommState(comdev_handle,&dcb) );

#if 0
  unsigned char buffer[500];
  for(register int i=0; i<sizeof(buffer); i++)
    buffer[i] = read_byte();
  FILE * fp = fopen("dump.1","wb");
  fwrite(buffer,sizeof(buffer),1,fp);
  fclose(fp);
#endif
}

			// Done with the port
LAPSWinIn::~LAPSWinIn(void)
{
  assert( CloseHandle(comdev_handle) );
}

			// Get a character from a serial link (or wait
			// until it becomes available)
int LAPSWinIn::wait_for_char(void)
{
   unsigned char read_c;
   DWORD no_chars_read = 1;
   unsigned long error;
//   message("\nClearcomm returned %d",ClearCommError(comdev_handle,&error,NULL));
//   message("\nerror was %x\n",error);
//   assert( ClearCommBreak(comdev_handle) );
   if( !ReadFile(comdev_handle,&read_c,1,&no_chars_read,NULL) )
     throw rc_input_error(errno);
   if( no_chars_read == 0 )
     throw rc_timeout();
   return read_c;
}

#if 0
			// Check to see if there is anything worth reading
bool LAPSWinIn::is_char_available(void)
{
   COMSTAT  dupa;
   GetCommError(comdev_id, &dupa);
   return dupa.cbInQue > 0;
}
#endif
  			// The standard LAPIn interface
unsigned char LAPSWinIn::read_byte(void)
{
  return wait_for_char();
}

void LAPSWinIn::look_for_start_of_packet(void)
{
  COMMTIMEOUTS no_timeouts = {0,0,0,0,0};  // means wait forever
  assert( SetCommTimeouts(comdev_handle,&no_timeouts) );
  register int c;
  while( (c=wait_for_char()) != SOH )
    ;
  COMMTIMEOUTS timeouts = {0,0,timeout,0,0};
  assert( SetCommTimeouts(comdev_handle,&timeouts) );
}

void LAPSWinIn::ending_packet(void)
{
}

