// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *	Reliable Broadcast over unreliable low-bandwidth (satellite)
 *			channels with no feedback
 *
 *		  Receiving and verifying file segments
 *
 * $Id: segment_in.cc,v 1.2 1999/09/24 19:46:20 oleg Exp oleg $
 *
 ************************************************************************
 */


#include "util.h"
#include "Segment.h"
#include "LAP.h"
#include <errno.h>

//------------------------------------------------------------------------
//		A stream to help de-compose a packet


		// All multi-byte quantities are read in the MSB first
		// byte order
SegmentInStream&
SegmentInStream::operator >> (unsigned short& a_short)
{
  a_short = take_byte();
  a_short = (a_short << 8) | take_byte();
  return *this;
}

SegmentInStream&
SegmentInStream::operator >> (unsigned long& a_long)
{
  a_long = take_byte();
  a_long = (a_long << 8) | take_byte();
  a_long = (a_long << 8) | take_byte();
  a_long = (a_long << 8) | take_byte();
  return *this;
}

SegmentInStream&
SegmentInStream::operator >> (const SegmentOutStream::Array& array)
{
  assert( array.size < Segment0::max_MTU );
  assert( array.size > 0 );
  for(register const char * p = array.ptr; p < array.ptr + array.size; )
    const_cast<char&>(*p++) = take_byte();
  return *this;
}

  			// Receive a byte from the link and into the
  			// buffer
void SegmentInStream::receive_byte(LAPIn& in_link)
{
  const unsigned char c = in_link.read_byte();
  curr_crc << c;
  *write_ptr++ = c;
  assert( write_ptr < buffer + sizeof(buffer) );
}


			// Receive a packet from the link layer, and check it
			// The constructor returns when the entire packet
			// has been received and checked for validity
			// If something goes awry, an exception is thrown

SegmentInStream::SegmentInStream(LAPIn& in_link)
	: read_ptr(buffer), write_ptr(buffer),
	  header(0,0,4)		// would be replaced with the real stuff later
{
  in_link.look_for_start_of_packet();
  
  			// Receive bytes of the header
  for(register int i=0; i<header.q_header_size(); i++)
    receive_byte(in_link);
  header.read(*this);	// and arrange them properly

  if( header.data_length >= sizeof(buffer) )
    throw rc_bad_packet(rc_bad_packet::size_error);
    
    			// get the body of the packet
  for(register int i=0; i<header.data_length; i++)
    receive_byte(in_link);

  CRC16 crc_received(in_link);
  if( !(crc_received == curr_crc) )
    throw rc_bad_packet(rc_bad_packet::CRC_error);

  in_link.ending_packet();
}

			// Dump the rest if the stream into a file
void SegmentInStream::write(FILE * fp)
{
  assert( read_ptr + header.data_length == write_ptr );
  if( fwrite(read_ptr,sizeof(unsigned char),header.data_length,fp) !=
      header.data_length )
    throw rc_file_error(errno);
  read_ptr = write_ptr;
}

//------------------------------------------------------------------------

				// Read the header of a segment
void SegmentHeader::read(SegmentInStream& in_packet)
{
  in_packet >> const_cast<ID&>(session_id)
  	    >> const_cast<unsigned long&>(seq_number)
  	    >> const_cast<unsigned char&>(data_length);
  if( data_length <= 1 )
    throw rc_bad_packet(rc_bad_packet::size_error);
}

  			// _Partially_ fill out Segment0 based on what found in
  			// some_packet
Segment0::Segment0(const SegmentHeader& some_packet)
	: SegmentHeader(0,some_packet.session_id,q_data_size()),
	  file_size(0), max_seq_no(0), 
	  MTU(some_packet.seq_number == 0 ? 0 : some_packet.data_length)
{
  file_name[0] = '\0';
  if( MTU >= max_MTU )
    throw rc_bad_packet(rc_bad_packet::size_error);
}

  			// Complete a partially finished
  			// Segment0 from a packet
void Segment0::complete(SegmentInStream& packet) throw (rc_bad_packet)
{
  const SegmentHeader& packet_header = packet.q_header();
  assert( packet_header.seq_number == 0 &&
  	  packet_header.session_id == session_id &&
  	  packet_header.data_length == data_length );
  assure( file_size == 0, "Segment0 was already ingested!" );
  
  packet >> const_cast<unsigned long&>(file_size)
  	 >> const_cast<unsigned char&>(MTU) >> max_seq_no
  	 >> byte_array(file_name,sizeof(file_name));
  
  if( MTU >= max_MTU || MTU == 0 || file_size == 0 || max_seq_no == 0 )
    throw rc_bad_packet(rc_bad_packet::bad_segment0);
}
