// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *	Reliable Broadcast over unreliable low-bandwidth (satellite)
 *			channels with no feedback
 *
 *			Sending out file segments
 * 
 * An env variable LAPMTU may be specified to tell the size of the
 * output packets (MTU), Segment0::default_MTU being the default (see below).
 *
 * $Id: segment_out.cc,v 1.2 1997/01/14 22:13:54 oleg Exp oleg $
 *
 ************************************************************************
 */


#include "util.h"
#include "Segment.h"
#include "LAP.h"

static const char LAPMTU_name [] = "LAPMTU";

//------------------------------------------------------------------------
//		A stream to help compose a packet


			// Create a packet stream, and tell the link layer
			// we're starting a packet
SegmentOutStream::SegmentOutStream(LAPOut& _out_link)
  : out_link(_out_link)
{
  out_link.begin_packet();
}

			// write the CRC and flush the packet to the link
SegmentOutStream::~SegmentOutStream(void)
{
  out_link << crc;
  out_link.finish_packet();
}

		// All multi-byte quantities are written in the MSB first
		// byte order
inline SegmentOutStream&
SegmentOutStream::operator << (const unsigned short a_short)
{
  write_byte(a_short>>8);
  write_byte(a_short);
  return *this;
}

inline SegmentOutStream&
SegmentOutStream::operator << (const unsigned long a_long)
{
  write_byte(a_long>>24);
  write_byte(a_long>>16);
  write_byte(a_long>>8);
  write_byte(a_long);
  return *this;
}


SegmentOutStream&
SegmentOutStream::operator << (const Array& array)
{
  assert( array.size < Segment0::max_MTU );
  assert( array.size > 0 );
  for(register const char * p = array.ptr; p < array.ptr + array.size; )
    write_byte(*p++);
  return *this;
}

//------------------------------------------------------------------------

				// Write the header of a segment
void SegmentHeader::write(SegmentOutStream& out_packet) const
{
  out_packet << session_id << seq_number << data_length;
}

			// Cut the base file name out of a full path
			// name
static inline const char * get_base_name(const char * full_file_name)
{
  const char * p = strrchr(full_file_name,'/');
  return p == 0 ? full_file_name : p + 1;
}

unsigned char Segment0::default_MTU = 40;

int Segment0::get_default_MTU(void)
{
  int MTU = atoi(xgetenv(LAPMTU_name,"0"));
  if( MTU == 0 )
    MTU = default_MTU;
  Logger::log("The transmission packet will be %d bytes long",MTU);
  assure( MTU > 4, "MTU must be at least 4");
  return MTU;
}

void Segment0::set_default_MTU(const int new_MTU)
{
  assert( new_MTU > 4 && new_MTU < max_MTU );
  default_MTU = new_MTU;
}

				// Fill in Segment 0
Segment0::Segment0(const char * full_file_name)
	: SegmentHeader(0,Random::get_id(),q_data_size()),
	  file_size(get_file_size(full_file_name)),
	  MTU(get_default_MTU())
{
  assure( file_size != 0, "Don't want to send a file of size 0" );
  xstrncpy(file_name,get_base_name(full_file_name),sizeof(file_name)-1);
  max_seq_no = (file_size + MTU - 1)/MTU;
}

				// Send the entire Segment0 packet
void Segment0::send(LAPOut& out_link) const
{
  SegmentOutStream(out_link) << (SegmentHeader)(*this) 
  	<< file_size << MTU << max_seq_no
  	<< byte_array(file_name,sizeof(file_name));
}

			// Log sending of Segment0, that is, the
  			// beginning of a broadcast
void Segment0::log(const char * title) const
{
  Logger::log("%s: session %d broadcasting '%s', size %d, "
  	      "in %d packets of size %d",
  	      title,session_id,file_name,file_size,max_seq_no,MTU);
}